/******************************************************************************/
/*                                                                            */
/*                        Test-Client fr ANS-Schnittstelle                   */
/*                                                                            */
/*                  Der Netzwerkkartentreiber fr HADES040/060                */
/*                                                                            */
/*  Modul:         ANS_TST.PRG                                                */
/*  File:          ANS_TST.C                                                  */
/*  Teilaufgabe:   Speed-Testprogramm fr DECDriver und Ethernetkarte         */
/*  Beschreibung:  Es wird ein Ethernet-Frame generiert, und wiederholt       */
/*                 gesendet. Insgesamt werden dabei ca. 10MB bertragen.      */
/*                 Danach wird die effektive Geschwindigkeit von DECDriver    */
/*                 berechnet.                                                 */
/*  Bemerkung:     Dieses Programm soll in einfacher Weise die Verwendung     */
/*                 der ANS-Schnittstelle demonstrieren.                       */
/*  Funktionen:    main                                                       */
/*                 ans_init                                                   */
/*                 ans_receive                                                */
/*                 ans_status                                                 */
/*                                                                            */
/*  (c)1996-1997 Markus Fichtenbauer                             04.10.1997   */
/*                                                                            */
/******************************************************************************/
/* $Id$ */
/******************************************************************************/
/* $Log$ */
/******************************************************************************/
#include <portab.h>                   /* Portab-Deklarationen (ULONG,...)     */
#include <cookie.h>                   /* Cookie Jar Binding                   */
#include <stdio.h>                    /* Standard-I/O (sprintf,...)           */
#include <string.h>                   /* String- und Speichermanipulationen   */
#include <time.h>                     /* Zeitfunktionen (clock,...)           */
#include <tos.h>                      /* Betriebsystemfunktionen (Cconws,...) */
#include <ext.h>                      /* externe Funktionen (delay,...)       */
#include "ans.h"                      /* ANS-Schnittstellen-Definitionen      */

BOOL ans_init(void);                  /* ANS-Schnittstelle initialisieren     */
WORD ans_receive(WORD, UBYTE *);      /* Empfangs-Routine                     */
void ans_status (void);               /* ANS-Statusinformationen              */
void dec_status (void);               /* DEC-Statusinformationen              */

BOOL        packet = FALSE;           /* Paket empfangen ?                    */
UBYTE       *adresse = NULL;          /* Adresse des Pakets                   */
WORD        laenge = 0;               /* Laenge des Pakets                    */
DEC_Driver  *dec_info = NULL;         /* Pointer auf DECDriver-Infos          */
char  message[100];                   /* String fr Textausgaben mit sprintf  */

/******************************************************************************/
/*                                                                            */
/*  Unterprogramm:  BOOL ans_init (void)                                      */
/*                                                                            */
/*  Beschreibung:   Sucht nach dem Cookie fuer die ANS-Schnittstelle und      */
/*                  initialisiert sie in gegebenem Falle.                     */
/*                                                                            */
/*  Parameter:      Returnwert = TRUE: ANS gefunden                           */
/*                               FALSE: ANS nicht gefunden                    */
/*                                                                            */
/******************************************************************************/
BOOL ans_init (void)
{
  ULONG *ans_driver;                  /* Pointer auf ANS-Schnittstelle        */
  ULONG superstack;
  LONG  temp;
  BOOL  identify;

  identify = FALSE;
  superstack = Super(0L);                   /* Supervisor-Mode                */
  if (CK_JarInstalled())                    /* Cookie Jar vorhanden           */
  {
    if (CK_ReadJar(CK_PKT, &temp))          /* Paket-Driver-Cookie gefunden?  */
    {
      identify = TRUE;                      /* ANS-Driver bereits installiert */
      ans_driver = (ULONG *)temp;           /* Adresse der ANS-Schnittstelle  */
      (ULONG)ans_reset    = ans_driver[NETRESET];
      (ULONG)ans_open     = ans_driver[NETOPEN];
      (ULONG)ans_release  = ans_driver[NETRELEASE];
      (ULONG)ans_send     = ans_driver[NETSEND];
      (ULONG)ans_getadr   = ans_driver[NETGETADR];
      (ULONG)ans_info     = ans_driver[NETINFO];
      (ULONG)ans_pktalloc = ans_driver[NETPKTALLOC];
      (ULONG)ans_pktfree  = ans_driver[NETPKTFREE];
      (ULONG)ans_config   = ans_driver[NETCONFIG];
    }

    if (CK_ReadJar(CK_DECDRV, &temp))       /* DEC-Driver-Cookie gefunden?    */
      dec_info = (DEC_Driver *)temp;        /* Adresse der DEC-Treiberinfos   */
    else
      identify = FALSE;                     /* doch kein DECDriver            */
  }

  Super((void*)superstack);                 /* zurck in den User-Mode        */

  return identify;
}

/******************************************************************************/
/*                                                                            */
/*  Unterprogramm:  WORD ans_receive (WORD len, UBYTE *buf)                   */
/*                                                                            */
/*  Beschreibung:   Routine fr den Empfang eines Ethernet-Pakets             */
/*                  Diese Routine wird von DECDriver aufgerufen, wenn fr das */
/*                  amgemeldete Protokoll ein Frame empfangen wurde.          */
/*                                                                            */
/*  Parameter:      len        = Laenge des empfangenen Ethernet-Frames       */
/*                  *buf       = Adresse des empfangenen Ethernet-Frames      */
/*                  Returnwert = TRUE: Paket empfangen                        */
/*                                                                            */
/*  Bemerkung:      Die Routine wird im Empfangsinterrupt aufgerufen          */
/*                                                                            */
/******************************************************************************/
WORD ans_receive (WORD len, UBYTE *buf)
{
  packet = TRUE;                            /* wir haben ein Frame empfangen  */
  adresse = buf;                            /* Adresse des Frames             */
  laenge = len;                             /* Lnge des Frames               */

/* Hier kann man nun das Paket nach belieben verarbeiten                      */

  return TRUE;                              /* Paket korrekt verarbeitet      */
/* Liefern wir hier FALSE an DECDriver zurck, so verwirft DECDriver dieses   */
/* empfangene Frame. Das Frame gehrt damit wieder DECDriver => keine         */
/* Zugriffe mehr auf den Speicherbereich dieses Frames vom Client aus !!!     */
}

/******************************************************************************/
/*                                                                            */
/*  Unterprogramm:  void ans_status (void)                                    */
/*                                                                            */
/*  Beschreibung:   Gibt aktuelle Statistik der ANS-Schnittstelle aus.        */
/*                                                                            */
/*  Parameter:      keine                                                     */
/*                                                                            */
/******************************************************************************/
void ans_status (void)
{
  ANS_STATUS  buffer;                       /* Buffer fr die Statusinfos     */

/* Statistik-Daten der ANS-Schnittstelle auslesen                             */
  ans_info((WORD)sizeof(ANS_STATUS), &buffer);

  Cconws("\n\rANS-statistics:\n\r");
  Cconws("------------------------------------------\r\n");
  sprintf(message, "Anzahl der gesendeten Frames: %ld\r\n", buffer.ans_sent);
  Cconws(message);
  sprintf(message, "Anzahl der Gesamt-Sendefehler: %ld\r\n", buffer.ans_xmit);
  Cconws(message);
  sprintf(message, "Anzahl der Kollisionen: %ld\r\n", buffer.ans_coll);
  Cconws(message);
  sprintf(message, "Anzahl der verteilten Frames: %ld\r\n", buffer.ans_got);
  Cconws(message);
  sprintf(message, "Anzahl der empfangenen Frames: %ld\r\n", buffer.ans_rcvd);
  Cconws(message);
  sprintf(message, "Anzahl der verlorenen Frames: %ld\r\n", buffer.ans_miss);
  Cconws(message);
  sprintf(message, "Anzahl der CRC Fehler: %ld\r\n",  buffer.ans_crc);
  Cconws(message);
  sprintf(message, "Interrupt Status Register: %x\r\n", buffer.ans_err);
  Cconws(message);
  sprintf(message, "Anzahl der freien Frames: %d\r\n", buffer.ans_free);
  Cconws(message);
  sprintf(message, "Anzahl bearbeitete Interrupts: %ld\r\n", buffer.ans_intr);
  Cconws(message);
  Cconws("------------------------------------------\r\n");
}

/******************************************************************************/
/*                                                                            */
/*  Hauptprogramm:  int main (void)                                           */
/*                                                                            */
/*  Beschreibung:   Programmskelett fr ANS-Clients                           */
/*                                                                            */
/******************************************************************************/
int main(void)
{
  DATAGRAM *frame;                          /* Aufbau eines Ethernet-Frames   */
  UBYTE eth_adr[10];                        /* Buffer fr Ethernetadresse     */
  WORD  handle;                             /* ANS-Handle                     */
  LONG  start, stop;                        /* Zeitmessung                    */
  ULONG i, rate;                            /* Laufvariable, Transferrate     */

  Cconws("\rTest of DEC-Ethernetcard via Loopback-Connector\r\n");
  Cconws("(c)04.10.97 by Markus Fichtenbauer\r\n\n");

  if (!ans_init())                          /* ANS-Schnittstelle vorhanden    */
  {
    Cconws("ANS not installed !!!\r\n");
    Cconws("Press any key\r\n");
    Cconin();
    return 0;                               /* ohne ANS geht gar nix          */
  }

  if (dec_info->drv_id == DEC_ANS)          /* DECDriver ID auslesen          */
  {
    sprintf(message, "DECDriver V%s for ANS found\r\n", dec_info->drv_ver);
    Cconws(message);
  }
  else                                      /* leider nicht DECDriver for ANS */
  {
    Cconws("DECDriver for ANS not installed\r\n");
    Cconws("Press any key\r\n");
    Cconin();
    return 0;                               /* auch ohne DECDriver geht nix   */
  }

  ans_getadr(6, eth_adr);                   /* Ethernetadresse auslesen       */
  sprintf(message, "Ethernetaddress: %02x:%02x:%02x:%02x:%02x:%02x\r\n\n",
          eth_adr[0], eth_adr[1], eth_adr[2],
          eth_adr[3], eth_adr[4], eth_adr[5]);
  Cconws(message);

/* Zunchst schalten wir defaultmig den Sendeinterrupt aus. Das Senden      */
/* erfolgt damit per Polling und ist (noch) ein wenig schneller als ber      */
/* die Interruptsteuerung. Wird sich in Zukunft durch ein effizienteres       */
/* Frame-Queueing ndern.                                                     */
  ans_config(CNF_DEC_SNDI, FALSE);          /* Interrupt-Steuerung aus        */

/* Zum Testen kann man sich aussuchen, ob mit oder ohne Sendeinterrupt        */
/* gearbeitet werden soll.                                                    */
  Cconws("with (1) or without (0) interrupt-control at sending frames:");
  scanf("%ld", &i);
  
  if (i)
  {
    ans_config(CNF_DEC_SNDI, TRUE);         /* Sende-Interrupt einschalten    */
    Cconws("\r\ninterrupt-control is now switched on\r\n");
  } 
  
/* Jetzt melden wir uns beim DECDriver mit einem Loopback-Protokoll an.       */
/* Die Protokoll-Typen werden ber sogenannte RFC's (Request for Comments)    */
/* definiert (z.B. IP, ARP, RARP usw.). Fr Loopback gibt's nichts            */
/* offizielles, aber dieses Programm dient ohnehin nur zu Testzwecken.        */
  Cconws("open loopback-protocol...\r\n");
  handle = ans_open(LOOP, ans_receive);     /* Protokoll anmelden             */

  if (handle < 0)                           /* Fehler aufgetreten ?           */
  {
    switch (handle)                         /* welcher Fehler ?               */
    {
      case EPROTAVAIL:                      /* kein Handle mehr verfgbar     */
        sprintf(message, "no more protocol-handles available\r\n");
      break;
      case EPROTUSED:                       /* Protokoll ist schon angemeldet */
        sprintf(message, "protocol is already installed\r\n");
      break;
      default:
        sprintf(message, "error nr. '%d' occured !\r\n", handle);
    }
    Cconws(message);
    Cconws("Press any key\n\r");
    Cconin();
    return 0;                               /* dann eben nicht ;-)            */
  }

/* Lesen wir mal die aktuelle Statistik ber die ANS-Schnittstelle aus        */
  ans_status();                             /* aktuelle ANS-Infos auslesen    */

/* Fr den Geschwindigkeitstest brauchen wir keine Empfangsframes, die wir    */
/* sonst ber den Loopbackstecker empfangen wrden. Daher schalten wir den    */
/* Receive-Teil des DEC-Controllers aus.                                      */
/* Achtung: wenn mehrere Clients gleichzeitig DECDriver verwenden, kann das   */
/* natrlich Probleme bereiten. Fr unseren Speed-Test aber egal...           */
/* Wenn sich ein Protokoll bei DECDriver anmeldet, wir automatisch der        */
/* Receiver gestartet (wenn er ohnehin nicht schon luft). Das bedeutet, da  */
/* wir am Ende des Programms den Receiver nicht explizit starten mssen.      */
  Cconws("disable receiver\r\n");
  ans_config(CNF_DEC_RCV, 0);               /* Receiver ausschalten           */

/* Jetzt fordern wir von DECDriver Speicher fr ein Sendeframe an             */
  Cconws("allocate memory for sending frame...\r\n");
  frame = ans_pktalloc(LOOP);               /* Speicher anfordern             */
  sprintf(message, "frame begins at %lx\r\n", frame);
  Cconws(message);

/* Jetzt knnen wir das Ethernet-Frame mit Daten fllen                       */
  Cconws("building frame and sending...\r\n");
  memset(frame->dst, 0xff, 6);              /* an die Broadcastadresse senden */
  memcpy(frame->src, eth_adr, 6);           /* eigene Ethernet-Adresse        */
  frame->typ = 0x9000;                      /* Ethernet-Typ Loopback          */
  memset(frame->data, 0x55, 1500);          /* Test-Daten                     */

  sprintf(message, "sending 10MB of data now...please wait!\r\n");
  Cconws(message);
  start = clock();                          /* Zeitmessung starten            */

  for (i = 0; i < 6990; i++)                /* soviele Frames fr 10MB        */
  {
/* Jetzt knnen wir endlich Senden. Die Framelnge ergibt sich aus max. 1500  */
/* Bytes an Nutzdaten und 14 Bytes des Headers (2*6 Bytes Adr. + 2 Bytes Typ) */
/* Um die max. Geschwindigkeit des Treibers alleine zu testen, schicken wir   */
/* immer wieder das gleiche Frame, um das Ergebnis durch das Fllen nicht zu  */
/* verflschen (und damit die Geschwindigkeit des Clients mitzumessen!)       */
    handle = ans_send(1514, (BYTE *)frame); /* Frame senden                   */

    if (handle < 0)                         /* Fehler aufgetreten ?           */
    {
      switch (handle)                       /* welcher Fehler ?               */
      {
        case EINIT:                         /* kein freier Sendedescriptor    */
          sprintf(message, "\nno descriptor for sending frame available\r\n");
        break;
        case EPARAM:                        /* Frame-Adresse unplausibel      */
          sprintf(message, "\naddress of frame (range/alignment) not plausible)\r\n");
        break;
        case EPKTLEN:                       /* Frame-Lnge zu gro            */
          sprintf(message, "\nframe length greater than 1514 Bytes\r\n");
        break;
        case ECOLLISION:                    /* Kollisionen aufgetreten        */
          sprintf(message, "\nexcessive collisions occured during sending\r\n");
        break;
        case ETIMEOUT:                      /* Timeout beim Senden            */
          sprintf(message, "\ntimeout during sending of frame\r\n");
        break;
        default:
          sprintf(message, "\nerror nr. '%d' occured !\r\n", handle);
      }
      Cconws(message);
    }
  }
  stop = clock();                           /* Zeitmessung stoppen            */

/* Das Frame mssen wir nun wieder freigeben, da wir es nicht mehr brauchen,  */
/* und es sonst nicht mehr in die Queue zurckkehren kann.                    */
  Cconws("\r\nfree memory of allocated frame...\r\n");
  ans_pktfree(frame);                       /* Speicher des Frames freigeben  */

/* Der Geschwindigkeitstest ist nun beendet und wir knnen das Loopback-      */
/* Protokoll nun auch wieder abmelden.                                        */
  Cconws("release loopback-protocol...\r\n");
  handle = ans_release(LOOP);               /* Loopback-Protokoll abmelden    */

  if (handle < 0)                           /* Fehler aufgetreten ?           */
    Cconws("protocol not found / not deinstalled\r\n");

  ans_status();                             /* aktuelle ANS-Infos auslesen    */

/* Berechnung der Transferrate beim Senden:                                   */
/* Es wurden i Frames gesendet mit jeweils 1500 Nutzdatenbytes                */
/* Die Funktion liefert CLK_TCK  Ticks pro Sekunde.                           */
/* Zuletzt wird auf kB normiert.                                              */
/* sprintf(message, "\nstart = %ld, stop = %ld \r\n\n", start, stop);         */
/* Cconws(message);                                                           */
  rate = (i*1500*CLK_TCK)/(1024*(stop-start));
  sprintf(message, "DECDriver sending speed: %ld kB/s\r\n", rate);
  Cconws(message);

/* Und jetzt knnen wir auch wieder auf die default-Einstellungen bezglich   */
/* Interrupt-Steuerung zurckschalten (und das ist 'ausschalten').            */
  ans_config(CNF_DEC_SNDI, FALSE);        /* Interrupt-Steuerung ausschalten  */

  Cconws("Press any key\n\r");
  Cconin();                               /* auf Tastendruck warten           */
  return 0;                               /* Programm - Ende                  */
}
