/******************************************************************************/
/*                                                                            */
/*                                DECDriver V0.95                             */
/*                                                                            */
/*                  Der Netzwerkkartentreiber fr HADES040/060                */
/*                                                                            */
/*  File:          ANS.H                                                      */
/*  Teilaufgabe:   Treiber fr Ethernetkarten mit DEC2104x LAN Controller     */
/*  Beschreibung:  Deklarationen bezglich DECDriver und ANS-Schnittstelle    */
/*                                                                            */
/*  (c)1996-1997 Markus Fichtenbauer                             28.09.1997   */
/*                                                                            */
/******************************************************************************/
/* $Id$ */
/******************************************************************************/
/* $Log$ */
/******************************************************************************/

/******************************************************************************/
/*                         Atari Networking System                            */
/******************************************************************************/
/*                          Cookie - Definitionen                             */
/******************************************************************************/
#define CK_PKT    0x5F504B54L     /* _PKT:  Paket Driver Cookie               */
#define CK_ICST   0x49435354L     /* ICST:  INETCUST Cookie                   */
#define CK_DECDRV 0x44454344L     /* DECD:  DECDriver Cookie                  */

/******************************************************************************/
/*                    Ethernet-spezifische Definitionen                       */
/******************************************************************************/
#define FRAME_SIZE_MIN    60      /* Mindestanzahl an Bytes fr ein Frame     */
#define FRAME_SIZE        1520    /* Anzahl der Bytes eines Frame-Buffers     */

#define IP                0x800   /* Ethernet-Type fr Internet Protocol      */
#define ARP               0x806   /* Type fr Adress Resolution Protocol      */
#define RARP              0x8035  /* Type fr Reverse ARP                     */
#define LOOP              0x9000  /* Type fr Loopback Interface              */

typedef struct                    /* Aufbau eines Ethernet-Frames             */
{
  UBYTE dst[6];                   /* Destination-Adresse                      */
  UBYTE src[6];                   /* Source-Adresse                           */
  UWORD typ;                      /* Ethernet-Type bzw. Laengeninfo           */
  UBYTE data[1500];               /* Nutzdaten (46...1500 Bytes)              */
} DATAGRAM;

/******************************************************************************/
/*                    ANS-Zugriffe ber den _PKT - Cookie                     */
/******************************************************************************/
#define NETRESET          0       /* Netzwerk - Reset                         */
#define NETOPEN           1       /* Netzwerk - Protokoll anmelden            */
#define NETRELEASE        2       /* Netzwerk - Protokoll abmelden            */
#define NETSEND           3       /* Netzwerk - Paket senden                  */
#define NETGETADR         4       /* Netzwerk - Ethernetadresse lesen         */
#define NETINFO           5       /* Netzwerk - Info/Status lesen             */
#define NETPKTALLOC       6       /* Netzwerk - Paket allokieren              */
#define NETPKTFREE        7       /* Netzwerk - Paket freigeben               */
#define NETCONFIG         8       /* Netzwerk - Konfiguration                 */

/******************************************************************************/
/*                            ANS - Fehlermeldungen                           */
/******************************************************************************/
#define EPROTAVAIL        -1      /* kein Protokoll-Handle mehr verfgbar     */
#define EPROTUSED         -2      /* Protokoll bereits angemeldet             */
#define EHANDLE           -3      /* --- wird zur Zeit nicht verwendet ---    */
#define EPROTBUSY         -4      /* --- wird zur Zeit nicht verwendet ---    */
#define EINIT             -5      /* Fehler bei Initialisierung               */
#define EPARAM            -6      /* Fehler bei den Parametern                */
#define EPKTLEN           -7      /* Lnge des Frames ist zu gro             */
#define ECOLLISION        -8      /* Kollisionen beim Senden aufgetreten      */
#define ETIMEOUT          -9      /* Timeout beim Senden aufgetreten          */

/******************************************************************************/
/*                         ANS-Treiber - Infostruktur                         */
/******************************************************************************/
typedef struct                    /* ANS - Info/Status - Struktur             */
{
  ULONG ans_sent;                 /* Anzahl der gesendete Pakete              */
  ULONG ans_xmit;                 /* Anzahl der Sendefehler                   */
  ULONG ans_coll;                 /* Anzahl der Kollisionen                   */
  ULONG ans_got;                  /* Anzahl der bertragenen Pakete           */
  ULONG ans_rcvd;                 /* Anzahl der empfangenen Pakete            */
  ULONG ans_miss;                 /* Anzahl der verlorenen Pakete             */
  ULONG ans_crc;                  /* Anzahl der CRC Fehler                    */
  UWORD ans_err;                  /* Genereller Status                        */
  UWORD ans_free;                 /* Anzahl der freien Pakete                 */
  ULONG ans_intr;                 /* Anzahl der bearbeiteten Interrupts       */
} ANS_STATUS;

/******************************************************************************/
/*                         ANS Funktions-Prototypen                           */
/******************************************************************************/
WORD (*ans_reset)(void) = NULL;
WORD (*ans_open)(WORD, WORD (*)(WORD, UBYTE *)) = NULL;
WORD (*ans_release)(WORD) = NULL;
WORD (*ans_send)(WORD, BYTE *) = NULL;
WORD (*ans_getadr)(WORD, UBYTE *) = NULL;
WORD (*ans_info)(WORD, ANS_STATUS *) = NULL;
DATAGRAM *(*ans_pktalloc)(UWORD) = NULL;
WORD (*ans_pktfree)(DATAGRAM *) = NULL;
WORD (*ans_config)(WORD, ULONG) = NULL;

/******************************************************************************/
/*                                 DECDriver                                  */
/******************************************************************************/
/*                           DECDriver - Typkennungen                         */
/******************************************************************************/
#define DEC_ANS           1       /* DECDriver fr die ANS-Schnittstelle      */
#define DEC_MINT          2       /* DECDriver fr MiNT-Net                   */
#define DEC_STING         3       /* DECDriver fr STinG                      */
#define DEC_BLIP          4       /* DECDriver fr BLIP                       */

/******************************************************************************/
/*             DECDriver - Konfiguration ber ANS-Schnittstelle               */
/******************************************************************************/
#define CNF_DEC_RCV        1      /* Konf.DEC: Empfangsteil ein/aus           */
#define CNF_DEC_SND        2      /* Konf.DEC: Sendeteil ein/aus              */
#define CNF_DEC_SNDI       3      /* Konf.DEC: Sendeinterrupt ein/aus         */
#define CNF_DEC_MLT        4      /* Konf.DEC: Pass all Multicast ein/aus     */
#define CNF_DEC_PRM        5      /* Konf.DEC: Promiscous Mode ein/aus        */
#define CNF_DEC_APO        6      /* Konf.DEC: Transmit Automatic Polling     */
#define CNF_DEC_OPM        7      /* Konf.DEC: Operating Mode                 */
#define CNF_DEC_FDP        8      /* Konf.DEC: Full Duplex ein/aus            */
#define CNF_DEC_ETH        9      /* Konf.DEC: Ethernetadresse ndern         */

#define CNF_IP_ADR        30      /* Konf. IP: Internetadresse ndern         */
#define CNF_IP_SUB        31      /* Konf. IP: Anzahl der Subnetbits          */

#define CNF_TCP_WND       40      /* Konf.TCP: TCP Window Size                */

#define CNF_ANS_MEM       50      /* Konf.ANS: anzulegende Buffergre        */
#define CNF_ANS_USR       51      /* Konf.ANS: Username                       */
#define CNF_ANS_HST       52      /* Konf.ANS: File mit der Hosttabelle       */
#define CNF_ANS_PWD       53      /* Konf.ANS: File mit den Passwrtern       */
#define CNF_ANS_GMT       54      /* Konf.ANS: Zeitzonen-Offset               */
#define CNF_ANS_TZN       55      /* Konf.ANS: Name der Zeitzone              */
#define CNF_ANS_TSV       56      /* Konf.ANS: IP Adressen zweier Timeserver  */
#define CNF_ANS_NSV       57      /* Konf.ANS: IP Adressen dreier Nameserver  */
#define CNF_ANS_GTW       58      /* Konf.ANS: IP Adresse der Gateway         */
#define CNF_ANS_HNM       59      /* Konf.ANS: Name dieses Host-Rechners      */
#define CNF_ANS_DNM       60      /* Konf.ANS: Domain-Name dieses Hosts       */
#define CNF_ANS_MLH       61      /* Konf.ANS: Name des Mailhosts             */
#define CNF_ANS_LGH       62      /* Konf.ANS: Name des default Loginhosts    */
#define CNF_ANS_PRH       63      /* Konf.ANS: Name des Printer-Servers       */
#define CNF_ANS_PRD       64      /* Konf.ANS: Name des Printer-Devices       */
#define CNF_ANS_FTW       65      /* Konf.ANS: Gre des FTP-Buffers          */

/******************************************************************************/
/*                            DECDriver - Infostruktur                        */
/******************************************************************************/
typedef struct                    /* Struktur DEC_Driver                      */
{
  ULONG cbio_adr;                 /* Configuration Base I/O-Adresse           */
  UWORD device_id;                /* ID des LAN Controllers (DEC21040)        */
  UWORD vendor_id;                /* Hersteller des LAN Controllers (Digital) */
  UWORD subsys_id;                /* ID der Ethernet-Karte                    */
  UWORD subvnd_id;                /* Hersteller der Ethernet-Karte            */
  UBYTE eth_adr[6];               /* Ethernet-Adresse                         */
  UWORD int_line;                 /* Interrupt Pin / Line                     */
  UBYTE brcst_adr[6];             /* Broadcast-Adresse                        */
  UBYTE slot_nr;                  /* Nummer des PCI-Slot {1,2,3,4}            */
  UBYTE rev_nr;                   /* Revision-Nummer des LAN Controllers      */
  UBYTE exist_conn;               /* vorhandene  Connectors {AUI,BNC,UTP}     */
  UBYTE select_conn;              /* selektierter Netzwerk-Anschluss          */
  UBYTE used_conn;                /* aktuell verwendeter Netzwerk-Anschluss   */
  UBYTE drv_id;                   /* Typ-Kennung (ANS/MiNT/STinG/...)         */
  UBYTE drv_ver[5];               /* Version des DECDrivers                   */
  char  card_type[30];            /* SMC-Kartentyp ("SMC8432T")               */

  ULONG rd_base;                  /* Base-Adresse der Empfangs-Deskriptoren   */
  ULONG td_base;                  /* Base-Adresse der Sende-Deskriptoren      */
  ULONG su_base;                  /* Base-Adresse des Setup-Buffers           */
  ULONG fr_base;                  /* Base-Adresse der Frame-Buffers           */
  ULONG fr_last;                  /* Ende des Frame-Buffers                   */
  ULONG CSR[16];                  /* die einzelnen Werte der CSR - Register   */
  ULONG CSR13_init;               /* Initialwert fr CSR13 - Connectivity Reg.*/
  ULONG CSR14_init;               /* Initialwert fr CSR14 - Xmit/Recv Reg.   */
  ULONG CSR15_init;               /* Initialwert fr CSR15 - General Register */
  ULONG irq_status;               /* Status-Register beim letzten Interrupt   */
  ULONG irq_anz;                  /* Anzahl der bearbeiteten Interrupts       */

  ULONG rx_received;              /* Anzahl der empfangenen Frames            */
  ULONG rx_distributed;           /* Anzahl der verteilten Frames             */
  ULONG rx_missed;                /* Counter fr 'missed Frames'              */
  ULONG rx_overflow;              /* Counter fr 'FIFO overflow' errors       */
  ULONG rx_checksum;              /* Counter fr 'checksum' errors            */
  ULONG rx_dribbling;             /* Counter fr 'dribbling bits' errors      */
  ULONG rx_collision;             /* Counter fr 'collision seen' errors      */
  ULONG rx_length;                /* Counter fr 'frame too long' errors      */

  ULONG tx_sent;                  /* Anzahl der gesendeten Frames             */
  ULONG tx_errors;                /* Gesamtanzahl der Sende-Fehler            */
  ULONG tx_underflow;             /* Counter fr 'FIFO underflow' errors      */
  ULONG tx_linkfail;              /* Counter fr 'link fail' errors           */
  ULONG tx_late_coll;             /* Counter fr 'late collision' errors      */
  ULONG tx_no_carrier;            /* Counter fr 'no carrier' errors          */
  ULONG tx_lost_carrier;          /* Counter fr 'lost carrier' errors        */
  ULONG tx_jabber;                /* Counter fr 'jabber timeout' errors      */
  ULONG tx_deferred;              /* Counter fr 'deferred' errors            */
  ULONG tx_heartbeat;             /* Counter fr 'heartbeat fail' errors      */
  ULONG tx_collisions;            /* Anzahl der aufgetretenen Kollisionen     */
  ULONG tx_coll_one;              /* Anzahl der einfachen Kollisionen         */
  ULONG tx_coll_mult;             /* Anzahl der mehrfachen Kollisionen        */
  ULONG tx_coll_max;              /* Anz. der Kollisionen mit Abbruch bei 16  */

  ULONG dec_config;               /* Konfigurationsleiste des DEC-Treibers    */
} DEC_Driver;

/******************************************************************************/
/*                                  INETCUST                                  */
/******************************************************************************/
#define MAXUSERNAME       16      /* max. Lnge eines Namens                  */
#define MAXNAME           64      /* max. Lnge eines Domainnamens            */
#define MAXFILENAME       64      /* max. Lnge eines Filenamens              */
#define MAXHOSTNAME       16      /* max. Lnge eines Hostnamens              */

typedef struct                    /* INETCUST - Struktur                      */
{
  ULONG magic;                    /* Lnge der INETCUST-Struktur              */
  UBYTE haddr[6];                 /* Ethernetadresse der SMC-Karte            */
  ULONG inaddr;                   /* eigene Internet-Adresse (IP)             */
  UWORD subnetbits;               /* Anzahl der Subnet-Bits                   */
  UWORD tcpwnd;                   /* TCP Window Size                          */
  UWORD tcplowater;               /* TCP Schwelle fr ???????                 */
  ULONG netmem;                   /* anzulegende Buffergre                  */
  BYTE  username[MAXUSERNAME];    /* Username                                 */
  BYTE  hosts[MAXFILENAME];       /* File mit der Hosttabelle                 */
  BYTE  passwd[MAXFILENAME];      /* File mit den Passwrtern                 */
  UWORD gmtoffs;                  /* Zeitzonen-Offset                         */
  BYTE  tzname[6];                /* Name der Zeitzone                        */
  ULONG timeserver[2];            /* IP Adressen zweier Timeserver            */
  ULONG nameserver[3];            /* IP Adressen dreier Nameservers           */
  ULONG gateway;                  /* IP Adresse der Gateway                   */
  BYTE  hostname[MAXHOSTNAME];    /* Name dieses Host-Rechners                */
  BYTE  domainname[MAXNAME];      /* Domainname dieses Host-Rechners          */
  BYTE  mailhost[MAXNAME];        /* Name des Mailhosts                       */
  BYTE  loginhost[MAXNAME];       /* Name des default Loginhosts              */
  BYTE  printhost[MAXNAME];       /* Name des Printer-Servers                 */
  BYTE  printdev[MAXHOSTNAME];    /* Name des Printer Devices                 */
  ULONG ftpwin;                   /* Gre des FTP - Buffers                  */
} INETCUST;

/******************************************************************************/
/*                          Deklarationen zum Debuggen                        */
/******************************************************************************/
#define DEBUG_SCREEN

#ifdef DEBUG_MODEM
  #define TRACEB(flag)    sprintf(tr_buffer,#flag "=%02x\n\r",flag);\
                          dpy=tr_buffer;while(*dpy)(Bconout(6,*dpy++));
  #define TRACEW(flag)    sprintf(tr_buffer,#flag "=%04x\n\r",flag);\
                          dpy=tr_buffer;while(*dpy)(Bconout(6,*dpy++));
  #define TRACEL(flag)    sprintf(tr_buffer,#flag "=%p\n\r",flag);\
                          dpy=tr_buffer;while(*dpy)(Bconout(6,*dpy++));
  #define TRACES(flag)    sprintf(tr_buffer,#flag "=%s\n\r",flag);\
                          dpy=tr_buffer;while(*dpy)(Bconout(6,*dpy++));
  #define TRACET(x)       dpy=x;while(*dpy)(Bconout(6,*dpy++));
#endif

#ifdef DEBUG_SCREEN
  #define TRACEB(flag)    printf(#flag "=%02x\n",flag);
  #define TRACEW(flag)    printf(#flag "=%04x\n",flag);
  #define TRACEL(flag)    printf(#flag "=%p\n",flag);
  #define TRACES(flag)    printf(#flag "=%s\n",flag);
  #define TRACET(flag)    printf(#flag "\n");
#endif
