/******************************************************************************/
/*                                                                            */
/*                        Test-Client fr ANS-Schnittstelle                   */
/*                                                                            */
/*                  Der Netzwerkkartentreiber fr HADES040/060                */
/*                                                                            */
/*  Modul:         ANS_TST.PRG                                                */
/*  File:          ANS_TST.C                                                  */
/*  Teilaufgabe:   Speed-Testprogramm fr DECDriver und Ethernetkarte         */
/*  Beschreibung:  Es wird ein Ethernet-Frame generiert, gesendet und ber    */
/*                 den Loopback-Stecker gleich wieder empfangen. Dazu wird    */
/*                 das Frame an sich selbst addressiert.                      */
/*  Bemerkung:     Dieses Programm soll in einfacher Weise die Verwendung     */
/*                 der ANS-Schnittstelle demonstrieren.                       */
/*  Funktionen:    main                                                       */
/*                 ans_init                                                   */
/*                 ans_receive                                                */
/*                 ans_status                                                 */
/*                 dec_status                                                 */
/*                                                                            */
/*  (c)1996-1997 Markus Fichtenbauer                             04.10.1997   */
/*                                                                            */
/******************************************************************************/
/* $Id$ */
/******************************************************************************/
/* $Log$ */
/******************************************************************************/
#include <portab.h>                   /* Portab-Deklarationen (ULONG,...)     */
#include <cookie.h>                   /* Cookie Jar Binding                   */
#include <stdio.h>                    /* Standard-I/O (sprintf,...)           */
#include <string.h>                   /* String- und Speichermanipulationen   */
#include <time.h>                     /* Zeitfunktionen (clock,...)           */
#include <tos.h>                      /* Betriebsystemfunktionen (Cconws,...) */
#include <ext.h>                      /* externe Funktionen (delay,...)       */
#include "ans.h"                      /* ANS-Schnittstellen-Definitionen      */

BOOL ans_init(void);                  /* ANS-Schnittstelle initialisieren     */
WORD ans_receive(WORD, UBYTE *);      /* Empfangs-Routine                     */
void ans_status (void);               /* ANS-Statusinformationen              */
void dec_status (void);               /* DEC-Statusinformationen              */

BOOL        packet = FALSE;           /* Paket empfangen ?                    */
UBYTE       *adresse = NULL;          /* Adresse des Pakets                   */
WORD        laenge = 0;               /* Laenge des Pakets                    */
DEC_Driver  *dec_info = NULL;         /* Pointer auf DECDriver-Infos          */
char  message[100];                   /* String fr Textausgaben mit sprintf  */

/******************************************************************************/
/*                                                                            */
/*  Unterprogramm:  BOOL ans_init (void)                                      */
/*                                                                            */
/*  Beschreibung:   Sucht nach dem Cookie fuer die ANS-Schnittstelle und      */
/*                  initialisiert sie in gegebenem Falle.                     */
/*                                                                            */
/*  Parameter:      Returnwert = TRUE: ANS gefunden                           */
/*                               FALSE: ANS nicht gefunden                    */
/*                                                                            */
/******************************************************************************/
BOOL ans_init (void)
{
  ULONG *ans_driver;                  /* Pointer auf ANS-Schnittstelle        */
  ULONG superstack;
  LONG  temp;
  BOOL  identify;

  identify = FALSE;
  superstack = Super(0L);                   /* Supervisor-Mode                */
  if (CK_JarInstalled())                    /* Cookie Jar vorhanden           */
  {
    if (CK_ReadJar(CK_PKT, &temp))          /* Paket-Driver-Cookie gefunden?  */
    {
      identify = TRUE;                      /* ANS-Driver bereits installiert */
      ans_driver = (ULONG *)temp;           /* Adresse der ANS-Schnittstelle  */
      (ULONG)ans_reset    = ans_driver[NETRESET];
      (ULONG)ans_open     = ans_driver[NETOPEN];
      (ULONG)ans_release  = ans_driver[NETRELEASE];
      (ULONG)ans_send     = ans_driver[NETSEND];
      (ULONG)ans_getadr   = ans_driver[NETGETADR];
      (ULONG)ans_info     = ans_driver[NETINFO];
      (ULONG)ans_pktalloc = ans_driver[NETPKTALLOC];
      (ULONG)ans_pktfree  = ans_driver[NETPKTFREE];
      (ULONG)ans_config   = ans_driver[NETCONFIG];
    }

    if (CK_ReadJar(CK_DECDRV, &temp))       /* DEC-Driver-Cookie gefunden?    */
      dec_info = (DEC_Driver *)temp;        /* Adresse der DEC-Treiberinfos   */
    else
      identify = FALSE;                     /* doch kein DECDriver            */
  }

  Super((void*)superstack);                 /* zurck in den User-Mode        */

  return identify;
}

/******************************************************************************/
/*                                                                            */
/*  Unterprogramm:  WORD ans_receive (WORD len, UBYTE *buf)                   */
/*                                                                            */
/*  Beschreibung:   Routine fr den Empfang eines Ethernet-Pakets             */
/*                  Diese Routine wird von DECDriver aufgerufen, wenn fr das */
/*                  amgemeldete Protokoll ein Frame empfangen wurde.          */
/*                                                                            */
/*  Parameter:      len        = Laenge des empfangenen Ethernet-Frames       */
/*                  *buf       = Adresse des empfangenen Ethernet-Frames      */
/*                  Returnwert = TRUE: Paket empfangen                        */
/*                                                                            */
/*  Bemerkung:      Die Routine wird im Empfangsinterrupt aufgerufen          */
/*                                                                            */
/******************************************************************************/
WORD ans_receive (WORD len, UBYTE *buf)
{
  packet = TRUE;                            /* wir haben ein Frame empfangen  */
  adresse = buf;                            /* Adresse des Frames             */
  laenge = len;                             /* Lnge des Frames               */

/* Hier kann man nun das Paket nach belieben verarbeiten                      */
/* Wir wollen hier nichts mit dem Paket anstellen, sonder nur ber die        */
/* Variable packet signalisieren, da wir ein Paket empfangen haben.          */

  return TRUE;                              /* Paket korrekt verarbeitet      */
/* Liefern wir hier FALSE an DECDriver zurck, so verwirft DECDriver dieses   */
/* empfangene Frame. Das Frame gehrt damit wieder DECDriver => keine         */
/* Zugriffe mehr auf den Speicherbereich dieses Frames vom Client aus !!!     */
}

/******************************************************************************/
/*                                                                            */
/*  Unterprogramm:  void ans_status (void)                                    */
/*                                                                            */
/*  Beschreibung:   Gibt aktuelle Statistik der ANS-Schnittstelle aus.        */
/*                                                                            */
/*  Parameter:      keine                                                     */
/*                                                                            */
/******************************************************************************/
void ans_status (void)
{
  ANS_STATUS  buffer;                       /* Buffer fr die Statusinfos     */

/* Statistik-Daten der ANS-Schnittstelle auslesen                             */
  ans_info((WORD)sizeof(ANS_STATUS), &buffer);

  Cconws("\n\rANS-statistics:\n\r");
  Cconws("------------------------------------------\r\n");
  sprintf(message, "Anzahl der gesendeten Frames: %ld\r\n", buffer.ans_sent);
  Cconws(message);
  sprintf(message, "Anzahl der Gesamt-Sendefehler: %ld\r\n", buffer.ans_xmit);
  Cconws(message);
  sprintf(message, "Anzahl der Kollisionen: %ld\r\n", buffer.ans_coll);
  Cconws(message);
  sprintf(message, "Anzahl der verteilten Frames: %ld\r\n", buffer.ans_got);
  Cconws(message);
  sprintf(message, "Anzahl der empfangenen Frames: %ld\r\n", buffer.ans_rcvd);
  Cconws(message);
  sprintf(message, "Anzahl der verlorenen Frames: %ld\r\n", buffer.ans_miss);
  Cconws(message);
  sprintf(message, "Anzahl der CRC Fehler: %ld\r\n",  buffer.ans_crc);
  Cconws(message);
  sprintf(message, "Interrupt Status Register: %x\r\n", buffer.ans_err);
  Cconws(message);
  sprintf(message, "Anzahl der freien Frames: %d\r\n", buffer.ans_free);
  Cconws(message);
  sprintf(message, "Anzahl bearbeitete Interrupts: %ld\r\n", buffer.ans_intr);
  Cconws(message);
  Cconws("------------------------------------------\r\n");
}

/******************************************************************************/
/*                                                                            */
/*  Unterprogramm:  void dec_status (void)                                    */
/*                                                                            */
/*  Beschreibung:   Gibt aktuelle Statistik des DEC-Treibers aus.             */
/*                  Dieses Unterprogramm dient nur als Beispiel!              */
/*                                                                            */
/*  Parameter:      keine                                                     */
/*                                                                            */
/*  Bemerkung:      Dient nur als Beispiel, man kann saemtliche Infos von     */
/*                  DECDriver auslesen, die mit der Struktur DEC_Driver       */
/*                  in ANS.H definiert sind.                                  */
/*                                                                            */
/******************************************************************************/
void dec_status (void)
{
  sprintf(message, "Kartenname: %s\r\n", dec_info->card_type);
  Cconws(message);                          /* Name der Ethernetkarte         */
  sprintf(message, "CSR5: 0x%lx\r\n", dec_info->CSR[5]);
  Cconws(message);                          /* aktuelles CSR5-Register        */
  sprintf(message, "CSR7: 0x%lx\r\n", dec_info->CSR[7]);
  Cconws(message);                          /* aktuelles CSR7-Register        */
  sprintf(message, "CSR12: 0x%lx\r\n\n", dec_info->CSR[12]);
  Cconws(message);                          /* aktuelles CSR12-Register       */
  Cconws("Press any key\r\n");
  Cconin();                                 /* auf Tastendruck warten         */
}

/******************************************************************************/
/*                                                                            */
/*  Hauptprogramm:  int main (void)                                           */
/*                                                                            */
/*  Beschreibung:   Programmskelett fr ANS-Clients                           */
/*                                                                            */
/******************************************************************************/
int main(void)
{
  DATAGRAM *frame;                          /* Aufbau eines Ethernet-Frames   */
  UBYTE eth_adr[10];                        /* Buffer fr Ethernetadresse     */
  WORD  handle;                             /* ANS-Handle                     */
  LONG  timeout;                            /* Timeout fr Empfang            */
  
  Cconws("\rTest of DEC-Ethernetcard via Loopback-Connector\r\n");
  Cconws("(c)04.10.97 by Markus Fichtenbauer\r\n\n");

  if (!ans_init())                          /* ANS-Schnittstelle vorhanden    */
  {
    Cconws("ANS not installed !!!\r\n");
    Cconws("Press any key\r\n");
    Cconin();                               /* auf Tastendruck warten         */
    return 0;                               /* ohne ANS geht gar nix          */
  }

  if (dec_info->drv_id == DEC_ANS)          /* DECDriver ID auslesen          */
  {
    sprintf(message, "DECDriver V%s for ANS found\r\n", dec_info->drv_ver);
    Cconws(message);
  }
  else                                      /* leider nicht DECDriver for ANS */
  {
    Cconws("DECDriver for ANS not installed\r\n");
    Cconws("Press any key\r\n");
    Cconin();                               /* auf Tastendruck warten         */
    return 0;                               /* auch ohne DECDriver geht nix   */
  }

  ans_getadr(6, eth_adr);                   /* Ethernetadresse auslesen       */
  sprintf(message, "Ethernetaddress: %02x:%02x:%02x:%02x:%02x:%02x\r\n\n",
          eth_adr[0], eth_adr[1], eth_adr[2],
          eth_adr[3], eth_adr[4], eth_adr[5]);
  Cconws(message);

/* Jetzt wollen wir mal Informationen ber den DECDriver auslesen             */
  dec_status();

/* Zunchst schalten wir defaultmig den Sendeinterrupt aus. Das Senden      */
/* erfolgt damit per Polling und ist (noch) ein wenig schneller als ber      */
/* die Interruptsteuerung. Wird sich in Zukunft durch ein effizienteres       */
/* Frame-Queueing ndern.                                                     */
  ans_config(CNF_DEC_SNDI, FALSE);          /* Interrupt-Steuerung aus        */

/* Jetzt melden wir uns beim DECDriver mit einem Loopback-Protokoll an.       */
/* Die Protokoll-Typen werden ber sogenannte RFC's (Request for Comments)    */
/* definiert (z.B. IP, ARP, RARP usw.). Fr Loopback gibt's nichts            */
/* offizielles, aber dieses Programm dient ohnehin nur zu Testzwecken.        */
  Cconws("\ropen loopback-protocol...\r\n");
  handle = ans_open(LOOP, ans_receive);     /* Protokoll anmelden             */

  if (handle < 0)                           /* Fehler aufgetreten ?           */
  {
    switch (handle)                         /* welcher Fehler ?               */
    {
      case EPROTAVAIL:                      /* kein Handle mehr verfgbar     */
        sprintf(message, "no more protocol-handles available\r\n");
      break;
      case EPROTUSED:                       /* Protokoll ist schon angemeldet */
        sprintf(message, "protocol is already installed\r\n");
      break;
      default:
        sprintf(message, "error nr. '%d' occured !\r\n", handle);
    }
    Cconws(message);
    Cconws("Press any key\n\r");
    Cconin();                               /* auf Tastendruck warten         */
    return 0;                               /* dann eben nicht ;-)            */
  }

/* Lesen wir mal die aktuelle Statistik ber die ANS-Schnittstelle aus        */
  ans_status();                             /* aktuelle ANS-Infos auslesen    */
  Cconws("Press any key\n\r");
  Cconin();                                 /* auf Tastendruck warten         */

/* Jetzt fordern wir von DECDriver Speicher fr ein Sendeframe an             */
  Cconws("\rallocate memory for sending frame...\r\n");
  frame = ans_pktalloc(LOOP);               /* Speicher anfordern             */
  sprintf(message, "frame begins at %lx\r\n", frame);
  Cconws(message);

/* Jetzt knnen wir das Ethernet-Frame mit Daten fllen                       */
  Cconws("building frame and sending...\r\n");

  memset(frame->dst, 0xff, 6);              /* an die Broadcastadresse senden */
  memcpy(frame->src, eth_adr, 6);           /* eigene Ethernet-Adresse        */
  frame->typ = 0x9000;                      /* Ethernet-Typ Loopback          */
  memset(frame->data, 0xAA, 100);           /* Test-Daten                     */
  packet = FALSE;                           /* noch kein Paket empfangen      */

/* Jetzt knnen wir endlich Senden. Die Framelnge ergibt sich aus 100 Bytes  */
/* an Nutzdaten und 14 Bytes des Headers (2*6 Bytes Adresse + 2 Bytes Typ)    */
  handle = ans_send(114, (BYTE *)frame);    /* Frame senden                   */

  if (handle < 0)                           /* Fehler aufgetreten ?           */
  {
    switch (handle)                         /* welcher Fehler ?               */
    {
      case EINIT:                           /* kein freier Sendedescriptor    */
        sprintf(message, "\nno descriptor for sending frame available\r\n");
      break;
      case EPARAM:                          /* Frame-Adresse unplausibel      */
        sprintf(message, "\naddress of frame (range/alignment) not plausible)\r\n");
      break;
      case EPKTLEN:                         /* Frame-Lnge zu gro            */
        sprintf(message, "\nframe length greater than 1514 Bytes\r\n");
      break;
      case ECOLLISION:                      /* Kollisionen aufgetreten        */
        sprintf(message, "\nexcessive collisions occured during sending\r\n");
      break;
      case ETIMEOUT:                        /* Timeout beim Senden            */
        sprintf(message, "\ntimeout during sending of frame\r\n");
      break;
      default:
        sprintf(message, "\nerror nr. '%d' occured !\r\n", handle);
    }
    Cconws(message);
  }

/* Das Frame mssen wir nun wieder freigeben, da wir es nicht mehr brauchen,  */
/* und es sonst fr andere Clients nicht mehr zur Verfgung stehen wrde.     */
  Cconws("\r\nfree memory of allocated frame-buffer...\r\n");
  ans_pktfree(frame);                       /* Speicher des Frames freigeben  */

  Cconws("...waiting for looped back Ethernet-Frame\r\n");
  timeout = clock() + CLK_TCK*2;            /* Timeout auf 2000 ms (2sec)     */
  while (!packet && clock() < timeout);     /* Timeout oder Frame empfangen ? */

  if (clock() >= timeout)                   /* Timeout aufgetreten ?          */
    Cconws("no looped back Ethernet-Frame received !!!\r\n\n");

  if (packet)                               /* Frame empfangen ?              */
  {
    Cconws("looped back Ethernet-Frame received, Ethernetcard works !!!\r\n");
    frame = (DATAGRAM *)adresse;            /* Adresse des empfangenen Frames */
    sprintf(message, "frame begins at %lx\r\n", frame);
    Cconws(message);                        /* Adresse des Frames ausgeben    */
    sprintf(message, "length of looped back frame: %d Bytes\r\n", laenge);
    Cconws(message);                        /* Lnge des Frames ausgeben      */
    sprintf(message, "Ethernetaddress(Destination): %02x:%02x:%02x:%02x:%02x:%02x\r\n",
            frame->dst[0], frame->dst[1], frame->dst[2],
            frame->dst[3], frame->dst[4], frame->dst[5]);
    Cconws(message);                        /* Destination-Adresse ausgeben   */
    sprintf(message, "Ethernetaddress(Source): %02x:%02x:%02x:%02x:%02x:%02x\r\n",
            frame->src[0], frame->src[1], frame->src[2],
            frame->src[3], frame->src[4], frame->src[5]);
    Cconws(message);                        /* Source-Adresse ausgeben        */
    sprintf(message, "Ethernet-type: %x\r\n", frame->typ);
    Cconws(message);                        /* Ethernet-Type ausgeben         */
    sprintf(message, "Data: %02x %02x %02x %02x %02x ...\r\n\n",
            frame->data[0], frame->data[1], frame->data[2],
            frame->data[3], frame->data[4]);
    Cconws(message);                        /* Die ersten Datenbytes ausgeben */

/* Das Frame mssen wir nun wieder freigeben, da wir es nicht mehr brauchen,  */
/* und es sonst nicht mehr in die Queue zurckkehren kann.                    */
    Cconws("free memory of received frame...\r\n");
    ans_pktfree(frame);                     /* Speicher des Frames freigeben  */
  }

  ans_status();                             /* aktuelle ANS-Infos auslesen    */

/* Der Test ist nun beendet und wir knnen das Loopback- Protokoll nun wieder */
/* abmelden.                                                                  */
  Cconws("release loopback-protocol...\r\n");
  handle = ans_release(LOOP);               /* Loopback-Protokoll abmelden    */

  if (handle < 0)                           /* Fehler aufgetreten ?           */
    Cconws("protocol not found / not deinstalled\r\n");

  Cconws("Press any key\n\r");
  Cconin();                               /* auf Tastendruck warten           */
  return 0;                               /* Programm - Ende                  */
}
